package TVListings::show;

use strict;
use warnings;

use Apache::Constants qw/:http/;

use POSIX;
use Date::Parse;
use CGI qw/escapeHTML/;

use TVListings::Config;
use TVListings::DB qw/:default SQL_PINFO SQL_CNAME/;
use TVListings::Links;

our %patt_renderers = (
	title => sub {'<h1>' . imdbtlink($_[1]{patt_value}) . '</h1>'},
	'sub-title' => sub {'<h2>' . escapeHTML($_[1]{patt_value}) . '</h2>'},
	desc => sub {'<b>Description:</b> ' . escapeHTML($_[1]{patt_value})
		. '<br/>'},
	category => [sub {'<b>Category:</b> ' . join(', ',
		map {escapeHTML($_->{patt_value})} @{$_[0]}) . '<br/>'}],
	'episode-num' => sub {'<b>Episode Number (' . escapeHTML($_[0]) . '):</b> '
		. escapeHTML($_[1]{patt_value}) . '<br/>'},
	rating => [sub {
		my $o = '';
		my @nonadv = sort grep {$_ ne 'advisory'} keys %{$_[0]};
		$o .= '<b>Rating:</b> ' . join(', ',
			map {escapeHTML("$_:$_[0]{$_}[0]{patt_value}")} @nonadv) . '<br/>'
				if @nonadv;
		$o .= '<b>Advisory:</b> ' . join(', ', map {escapeHTML($_->{patt_value})}
			@{$_[0]{advisory}}) . '<br/>' if $_[0]{advisory};
		return $o;
	}],
	'star-rating' => sub {'<b>Stars:</b> ' . escapeHTML($_[1]{patt_value})
		. '<br/>'},
	subtitles => sub {'<b>Subtitles:</b> ' . escapeHTML($_[1]{patt_value})
		. '<br/>'},
	date => sub {'<b>Original Air Date:</b> ' . escapeHTML($_[1]{patt_value})
		. '<br/>'},
	credits => [sub {
		my $o = '<b>Credits:</b><br/>';
		for my $credit (sort keys %{$_[0]}) {
			(my $cname = $credit) =~ s/^(.)/uc($1)/e;
			$cname .= 's' if $#{$_[0]{$credit}} > 0;
			$o .= '<div style="margin-left:1em"><b><i>' . escapeHTML($cname)
				. ':</i></b> ' . join(', ', map {personlink($_->{patt_value})}
				@{$_[0]{$credit}}) . '</div>';
		}
		return $o;
	}],
	audio => sub {sprintf '<b>Audio:</b> %s: %s<br/>', escapeHTML($_[0]),
		escapeHTML($_[1]{patt_value})},
	video => sub {sprintf '<b>Video:</b> %s: %s<br/>', escapeHTML($_[0]),
		escapeHTML($_[1]{patt_value})},
	length => sub {'<b>Running Time:</b> ' . escapeHTML($_[1]{patt_value})
		. ' minutes<br/>'},
	
);
sub render {
	my ($class, $r, $q, $dbh) = @_;
	
	# pagehead gets put in later
	my $out = '<div class="showinfo">';
	
	if (!$q->param('prg')) {
		$out = pagehead($r, $q, "TV Show Details")
			. '<i>No prorgram specified</i><br/></div>' . pagefoot($r, $q);
		return { status => HTTP_BAD_REQUEST, text => $out };
	}
	
	my $pisth = $dbh->prepare_cached(SQL_PINFO);
	$pisth->execute($q->param('prg'));
	my $pinfo = $pisth->fetchall_arrayref({});
	if ($#$pinfo > 0) {
		$out .= '<i>Multiple programs hit? Only showing first one</i><br/>';
	} elsif ($#$pinfo < 0) {
		$out = pagehead($r, $q, "TV Show Details")
			. '<i>No program found</i><br/></div>' . pagefoot($r, $q);
		return { status => HTTP_OK, text => $out };
	}
	$pinfo = $pinfo->[0];
	my $patts = get_patts($dbh, $pinfo->{prg_oid});
	my $cnsth = $dbh->prepare_cached(SQL_CNAME);
	$cnsth->execute($pinfo->{chan_oid});
	my $cinfo = $cnsth->fetchall_arrayref({})->[0];
	my $cname = "$cinfo->{chan_number} $cinfo->{chan_name}";
	my $cicon = get_cicon($dbh, $pinfo->{chan_oid});
	my ($iw, $ih) = @{$cicon}{qw/icn_width icn_height/};
	
	#FIXME: fetch icons for program and ratings? na_dd doesn't output any
	
	my $searchlink = '';
	my $prg_title = '';
	for my $att (qw/title sub-title desc category episode-num rating
			star-rating subtitles date credits audio video length/) {
		
		if ($att eq 'desc') {
			$out .= '<div style="margin-left:1em">';
			$out .= '<b>Channel:</b> ' . chanlink($r, $pinfo->{chan_oid}, $cname);
			$out .= chanlink($r, $pinfo->{chan_oid},
				qq{<img src="@{[ICONSBASE]}/$cicon->{icn_src_file}" }
				. qq{alt="@{[escapeHTML($cname)]}"}
				. (($iw and $ih) ? qq{ width="$iw" height="$ih"} : '')
				. '/>') if $cicon->{icn_src_file};
			$out .= '<br/>';
			my ($pstart, $pstop) = map {escapeHTML(strftime("%c", strptime($_)))}
				@{$pinfo}{qw/prg_start prg_stop/};
			$out .= "<b>Time:</b> $pstart - $pstop<br/>";
		} elsif ($att eq 'title') {
			$prg_title = $patts->{title}[0]{patt_value};
			$searchlink = searchlink($prg_title, "Search for other airings");
		}
		
		next if !$patts->{$att};
		
		if (ref($patt_renderers{$att}) eq 'ARRAY') {
			$out .= $patt_renderers{$att}[0]->($patts->{$att});
		} else {
			if (ref($patts->{$att}) eq 'HASH') {
				for my $satt (sort keys %{$patts->{$att}}) {
					$out .= $patt_renderers{$att}->($satt, $_)
						foreach @{$patts->{$att}{$satt}};
				}
			} else {
				$out .= $patt_renderers{$att}->(undef, $_) foreach @{$patts->{$att}};
			}
		}
			
		delete $patts->{$att};
	}
	
	# now that we have the prg title, prepend the header
	$out = pagehead($r, $q, "TV Show Details - $prg_title") . $out;
	
	$out .= "</div><p>$searchlink</p>";
	
	my @pintclass = get_cv_pint_class($dbh);
	my $pint_cur = get_prg_interest($dbh, $prg_title);
	my @bayes_details = get_bayes_details($dbh, $pinfo->{prg_oid});
	
	$out .= qq{<form method="post" action="./edit"><p>}
		. qq{<input type="hidden" name="prg_title" value=\"}
			# hack to make utf8 stuff pass through right, we convert the title
			# to octets
			. escapeHTML($prg_title) . qq{\"/>}
		. qq{<input type="hidden" name="prg_oid" value=\"}
			. escapeHTML($pinfo->{prg_oid}) . qq{\"/>}
		. qq{<input type="hidden" name="returl" value=\"}
#			. escapeHTML("./#chan_$cinfo->{chan_number}_$cinfo->{chan_name}")
			. escapeHTML("./#top")
			. qq{\"/>}
		. qq{<b>Interest:</b> <select name="pint_class">}
		. qq{<option value="">&lt;none&gt;</option>}
		. join('', map {
				qq{<option value="$_"}
				. ($_ eq escapeHTML($pint_cur) ? qq{ selected="selected"} : '')
				. qq{>$_</option>}
			} map {escapeHTML($_)} @pintclass)
		. qq{</select> }
		. qq{<input type="submit" name="action" value="Flag it"/> }
		. qq{<input type="submit" name="action" value="Learn it"/> }
		. qq{</p></form>};
	
	$out .= q{<table class="prg_bayes">}
		. q{<caption>Bayes Info</caption>}
		. q{<tr><th>Interest</th><th>Probability</th><th># Tokens</th></tr>};
	for my $bayes_info (@bayes_details) {
		$out .= q{<tr>} . join('', map {'<td>' . escapeHTML($_) . '</td>'}
			$bayes_info->{pint_class}, sprintf('%.4f', $bayes_info->{bayes_prob}),
			$bayes_info->{num_toks}) . '</tr>';
	}
	$out .= q{</table>};
	
	if (scalar keys %$patts) {
		$out .= 'Extra Info:<br/><table border="1">';
		
		sub outrow {
			return '<tr>' . join('',
				map {'<td>' . escapeHTML(defined $_ ? $_ : '') . '</td>'} @_)
				. "</tr>\n";
		}
		
		for my $att (sort keys %$patts) {
			if (ref($patts->{$att}) eq 'HASH') {
				foreach my $satt (sort keys %{$patts->{$att}}) {
					foreach my $av (@{$patts->{$att}{$satt}}) {
						$out .= outrow($att, $satt, @{$av}{qw/patt_lang patt_value/});
					}
				}
			} else {
				foreach my $av (@{$patts->{$att}}) {
					$out .= outrow($att, '', @{$av}{qw/patt_lang patt_value/});
				}
			}
		}
		
		$out .= '</table>';
	}
	
	$out .= '</div>';
	
	$out .= pagefoot($r, $q);
	
	return {
		status => HTTP_OK,
		text => $out,
	};
}

